// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v16.services;

import "google/ads/googleads/v16/resources/custom_interest.proto";
import "google/api/annotations.proto";
import "google/api/client.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/protobuf/field_mask.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V16.Services";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v16/services;services";
option java_multiple_files = true;
option java_outer_classname = "CustomInterestServiceProto";
option java_package = "com.google.ads.googleads.v16.services";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V16\\Services";
option ruby_package = "Google::Ads::GoogleAds::V16::Services";

// Proto file describing the Custom Interest service.

// Service to manage custom interests.
service CustomInterestService {
  option (google.api.default_host) = "googleads.googleapis.com";
  option (google.api.oauth_scopes) = "https://www.googleapis.com/auth/adwords";

  // Creates or updates custom interests. Operation statuses are returned.
  //
  // List of thrown errors:
  //   [AuthenticationError]()
  //   [AuthorizationError]()
  //   [CriterionError]()
  //   [CustomInterestError]()
  //   [HeaderError]()
  //   [InternalError]()
  //   [MutateError]()
  //   [PolicyViolationError]()
  //   [QuotaError]()
  //   [RequestError]()
  //   [StringLengthError]()
  rpc MutateCustomInterests(MutateCustomInterestsRequest)
      returns (MutateCustomInterestsResponse) {
    option (google.api.http) = {
      post: "/v16/customers/{customer_id=*}/customInterests:mutate"
      body: "*"
    };
    option (google.api.method_signature) = "customer_id,operations";
  }
}

// Request message for
// [CustomInterestService.MutateCustomInterests][google.ads.googleads.v16.services.CustomInterestService.MutateCustomInterests].
message MutateCustomInterestsRequest {
  // Required. The ID of the customer whose custom interests are being modified.
  string customer_id = 1 [(google.api.field_behavior) = REQUIRED];

  // Required. The list of operations to perform on individual custom interests.
  repeated CustomInterestOperation operations = 2
      [(google.api.field_behavior) = REQUIRED];

  // If true, the request is validated but not executed. Only errors are
  // returned, not results.
  bool validate_only = 4;
}

// A single operation (create, update) on a custom interest.
message CustomInterestOperation {
  // FieldMask that determines which resource fields are modified in an update.
  google.protobuf.FieldMask update_mask = 4;

  // The mutate operation.
  oneof operation {
    // Create operation: No resource name is expected for the new custom
    // interest.
    google.ads.googleads.v16.resources.CustomInterest create = 1;

    // Update operation: The custom interest is expected to have a valid
    // resource name.
    google.ads.googleads.v16.resources.CustomInterest update = 2;
  }
}

// Response message for custom interest mutate.
message MutateCustomInterestsResponse {
  // All results for the mutate.
  repeated MutateCustomInterestResult results = 2;
}

// The result for the custom interest mutate.
message MutateCustomInterestResult {
  // Returned for successful operations.
  string resource_name = 1 [(google.api.resource_reference) = {
    type: "googleads.googleapis.com/CustomInterest"
  }];
}
